class EventHandler(object):
    '''
        Abstract class for all EventHandler
        You only need to implement method proccess(self,signal,events,polling)
        You can use self.params to get parameters for your action
    '''

    def __init__(self, params, polling):
        self.params = params
        self.polling = polling

    def proccess(self, signal, events):
        '''
        Method need to be implemented
        accept events, with list of changed tags
        and polling object, that have methods getTag and setTag
        '''
        raise NameError("Method not implemented by derived class")

    def stop(self):
        '''
        Method need to be implemented
        In this method handler have to close all resource
        '''
        print "stop EventHandler " + str(self)



'''
Core handler
'''
from abstracthandler import AbstractHandler
from twisted.internet import reactor

import logging

logger = logging.getLogger('corehandler')


class corehandler(AbstractHandler):
    '''
    CoreHandler
    '''

    def __init__(self, parent=None, params={}):
        assert "configfile" in params, "no param configfile"
        self.listeners = {}
        AbstractHandler.__init__(self, parent, params)
        params = self.config[type(self).__name__]["params"]
        logger.info('Init core server')
        # if self.config[__name__].get("run", "1") == "1":
        #     self.start()

    def loadtags(self):
        for tag in self.config:
            self.settag(tag, self.config[tag].get("run", "1"))

    def _addhandler(self, classname, parent, params):
        if not classname in self.listeners:
            # try:
            _temp = __import__(
                classname, globals(), locals(), [classname])
            handler = eval(
                "_temp." + classname + "(parent, params)")
            self.listeners[classname] = handler
            # except ImportError, e:
            #     print e

    def _addhandlers(self, handlers):
        for tag in handlers:
            if tag != type(self).__name__:
                # try:
                classname = tag
                params = handlers[tag].get("params", {})
                parentname = handlers[tag].get("parent")
                if parentname == type(self).__name__:
                    parent = self
                else:
                    parent = self.listeners.get(parentname, None)
                self._addhandler(classname, parent, params)
                # except KeyError, e:
                #     print "No such param " + str(e)

    def runhandler(self, classname):
        if classname in self.listeners:
            self.listeners[classname].start()

    # def settag(self, tag, value):
        # self._settag(tag, value)

    def settag(self, tag, value):
        logger.debug("Setting tag %s to %s" % (tag, value))
        l = tag.split("_")
        # try:
        if len(l) == 2:
            if l[0] == type(self).__name__:
                if self._tags[l[1]] != value:
                    self._set_listeners(l[1], value)
                    AbstractHandler.settag(self, l[1], value)
            else:
                self.listeners[l[0]].settag(l[1], value)
        else:
            self._tags[tag] = value
        # except:
        #     logger.error(
        #         "Can't settag", exc_info=1)

    def _set_listeners(self, tag, value):
        if tag in self.listeners:
            if value:
                self.listeners[tag].start()
            else:
                self.listeners[tag].stop()

    def _gettag(self, tag):
        l = tag.split("_")
        if len(l) == 2:
            if l[0] == type(self).__name__:
                return self._tags[l[1]]
            else:
                return self.listeners[l[0]].gettag(l[1])
        else:
            return self._tags[tag]

    @property
    def tags(self):
        tagslist = {}
        for listener in self.listeners.values():
            for tag in listener.tags:
                tagslist[
                    "%s_%s" %
                    (listener.__class__.__name__, tag)] = listener.tags[tag]
        for tag in self._tags:
            tagslist["%s_%s" % (type(self).__name__, tag)] = self._tags[tag]
        return tagslist

    def stop(self):
        AbstractHandler.stop(self)
        #for listener in self.listeners:
            #self._set_listeners(listener, 0)
        reactor.stop()

    def start(self):
        # self.stopped = False
        logger.debug("RUN")
        self.settag(type(self).__name__, '1')
        self._addhandlers(self.config)
        for listener in self.listeners:
            if self._tags[listener] == '1':
                self.listeners[listener].start()
        reactor.run(installSignalHandlers=0)



import louie
from louie import dispatcher, TwistedDispatchPlugin
from config.configobj import ConfigObj
from datetime import datetime

louie.install_plugin(TwistedDispatchPlugin)

import sys
from os import path
import logging
import logging.handlers

logfiles_num = 5
logfile_size = 1048576
logger = logging.getLogger()
logger.setLevel(logging.DEBUG)
handler = logging.handlers.RotatingFileHandler(
    'smhs.log', maxBytes=logfile_size, backupCount=logfiles_num)
form = logging.Formatter(
    '%(asctime)s %(name)-12s %(levelname)s:%(message)s')
handler.setFormatter(form)
logger.addHandler(handler)

logger = logging.getLogger('abstracthandler')

class AbstractHandler(object):

    '''
    Abstractss class for all handlers
    '''

    def __init__(self, parent=None, params={}):
        if "configfile" in params:
            config_path = path.join(
                sys.prefix, 'etc/pysmhs', params['configfile'])
            self.config = ConfigObj(config_path, indent_type="\t")
        self.signal = self.__class__.__name__
        self.params = params
        self.stopped = True
        self.parent = parent
        self._tags = {}
        self.events = []
        logger.debug("Have params - " + str(self.params))
        logger.debug("Have parent - " + str(self.parent))
        self.loadtags()

    def process(self, signal, events):
        '''
        Method need to be implemented
        accept events, with list of changed tags
        '''
        pass

    def sendevents(self, tag, value):
        '''
        send event
        '''
        logger.debug(
            'Send tag: {} with value {} from {}'.format(
                tag, value, self.signal))
        dispatcher.send(signal=self.signal, event={tag:value})

    def settag(self, tag, value):
        '''
        Private method for settag
        set tag to value in tags
        Override if you need some action
        '''
        if self._tags[tag] != value:
            logger.debug('change tag {} to value {}'.format(tag, value))
            self._tags[tag] = value
            self.sendevents(tag, value)

    def gettag(self, tag):
        '''
        Private method for gettag
        get tag from tags
        Override if you need some action
        '''
        logger.debug("RETURN %s" % self._tags[tag])
        return self._tags[tag]

    @property
    def tags(self):
        if self.stopped:
            return {}
        return self._tags

    def loadtags(self):
        '''
        Load tags from config
        '''
        pass

    def stop(self):
        '''
        Stop handler
        '''
        self.stopped = True
        logger.info("Stop handler")
        dispatcher.disconnect(self.process, signal=self.params.get(
            "listensignals", dispatcher.Any))

    def start(self):
        '''
        Start handler
        '''
        self.stopped = False
        logger.info("Start handler")
        dispatcher.connect(self.process, signal=self.params.get(
            "listensignals", dispatcher.Any))



'''
Date handler
'''
from abstracthandler import AbstractHandler
from datetime import datetime, time
from twisted.internet.task import LoopingCall
from dateutil.parser import parse
from dateutil.rrule import *
import urllib2
import logging
from xml.dom import minidom

logger = logging.getLogger()


class datehandler(AbstractHandler):

    midnight = time(00, 00, 00)

    def updatedate(self):
        logger.debug("UPDATE")
        now = datetime.now().replace(microsecond=0)
        self._tags['date'] = now.strftime("%d.%m.%Y %H:%M:%S")
        self.checktags(now)
        if now.time() == self.midnight:
            self._tags['sunset'] = self.getsunset().strftime("%H:%M:%S")
            self._tags['sunrise'] = self.getsunrise().strftime("%H:%M:%S")
        if now.time().strftime("%H:%M:%S") == self._tags['sunset']:
            self.settag('issunset', '1')
        else:
            self.settag('issunset', '0')
        if now.time().strftime("%H:%M:%S") == self._tags['sunrise']:
            self.settag('issunrise', '1')
        else:
            self.settag('issunrise', '0')

    def loadtags(self):
        self._tags['sunset'] = self.getsunset().strftime("%H:%M:%S")
        self._tags['sunrise'] = self.getsunrise().strftime("%H:%M:%S")
        self._tags['issunset'] = '0'
        self._tags['issunrise'] = '0'
        self.rrules = {}
        for tag, params in self.config.items():
            startdate = parse(params["start"])
            enddate = parse(params["end"])
            untildate = parse(params["until"])
            freq = eval(params['freq'])
            startrr = rrule(
                freq, dtstart=startdate, until=untildate, cache=True)
            endrr = rrule(freq, dtstart=enddate, until=untildate, cache=True)
            self._tags[tag] = '0'
            tagrrule = self.rrules.setdefault(tag, {})
            tagrrule["startrr"] = startrr
            tagrrule["endrr"] = endrr

    def earthtool(self, param):
        now = datetime.now()
        xml = urllib2.urlopen(
            "http://www.earthtools.org/sun/54.32/36.16/" +
            str(now.day) + "/" +
            str(now.month) + "/+4/0").read()
        xmldoc = minidom.parseString(xml)
        itemlist = xmldoc.getElementsByTagName(param)
        return parse(itemlist[0].childNodes[0].nodeValue).time()

    def getsunset(self):
        sunset = time(hour=20, minute=00, second=0, microsecond=0)
        #  try:
            #  sunset = self.earthtool('sunset')
        #  except:
            #  logger.error("Cant get sunset", exc_info=1)
        return sunset

    def getsunrise(self):
        sunrise = time(hour=8, minute=00, second=0, microsecond=0)
        #  try:
            #  sunrise = self.earthtool('sunrise')
        #  except:
            #  logger.error("Cant get sunrise", exc_info=1)
        return sunrise

    def checktag(self, tag, dt):
        dt = dt.replace(second=0, microsecond=0)
        if dt in self.rrules[tag]["startrr"]:
            self.settag(tag, '1')
        elif dt in self.rrules[tag]["endrr"]:
            self.settag(tag, '0')

    def checktags(self, dt):
        for tag in self.rrules:
            self.checktag(tag, dt)

    def start(self):
        AbstractHandler.start(self)
        self.lc = LoopingCall(self.updatedate)
        self.lc.start(1)

    def stop(self):
        AbstractHandler.stop(self)
        if self.lc:
            self.lc.stop()



'''
Action Handler
'''
from abstracthandler import AbstractHandler
import time
import logging

logger = logging.getLogger()


class actionhandler(AbstractHandler):

    def __init__(self, parent=None, params={}):
        AbstractHandler.__init__(self, parent, params)
        self.actions = {'setter': self.setter,
                        'switcher': self.switcher,
                        'sleep': self.sleep}
        self.temp_tags = {}

    def process(self, signal, events):
        for event in events:
            logger.debug(event)
            self.temp_tags[event['tag']] = event['value']
            for tag, params in self.config.items():
                if 'condition' in params:
                    params = params.copy()
                    cond = params.pop('condition')
                    if event['tag'] in cond:
                        logger.debug("check cond %s" % cond)
                        try:
                            logger.debug("eval %s" % eval(cond))
                            if eval(cond):
                                self._settag(tag, '1')
                        except Exception, e:
                            logger.error(
                                "Error(%s) while eval(%s)" % (e, cond))
        self.temp_tags = {}
        logger.debug('End of process')

    def gettag(self, tag):
        if tag in self.temp_tags:
            return int(self.temp_tags[tag])
        return AbstractHandler.gettag(self, tag)

    def _settag(self, tag, value):
        AbstractHandler._settag(self, tag, value)
        if str(value) == '1':
            self.run_action(tag)

    def run_action(self, tag):
        params = self.config[tag].copy()
        logger.debug('have actions %s' % sorted(params))
        if 'condition' in params:
            params.pop('condition')
        for i in sorted(params):
            logger.debug('Now in %s' % i)
            l = i.split(".")
            if len(l) == 2:
                action = l[1]
                param = params[i]
                logger.debug('Call method %s' % i)
                self.actions.get(action, None)(param)
            else:
                logger.debug(
                    'Wrong action name and order - %s' % i)
        logger.debug('have after actions %s' % sorted(params))
        self._settag(tag, '0')

    def loadtags(self):
        for tag in self.config:
            self._tags[tag] = '0'

    def switcher(self, params):
        for tag in params:
            self._inverttag(tag)

    def setter(self, params):
        for tag, value in params.items():
            self.settag(tag, value)

    def _inverttag(self, tag):
        if str(self.gettag(tag)) == '0':
            self.settag(tag, '1')
        else:
            self.settag(tag, '0')

    def sleep(self, params):
        logger.debug('before timeout')
        time.sleep(float(params.get('timeout', 1)))
        logger.debug('after timeout')



from setuptools import setup, find_packages
import sys

setup(
    name="pysmhs",
    version="1.0.4",
    packages=find_packages(exclude=["test"]),
    data_files=[
        ('etc/pysmhs',
         ['config/coreconfig.txt', 'config/actions.txt',
          'config/dateconfig.txt', 'config/tags_config.txt'])],
    include_package_data=True,
    author="Aborilov Pavel",
    description="Smart House Control System",
    author_email = 'aborilov@gmail.com',
    url = 'http://www.pysmhs.org/',
    download_url = 'https://github.com/aborilov/pysmhs/archive/develop.zip',
    install_requires=[
        "Jinja2",
        "Twisted",
        "pymodbus",
        "pyserial",
        "python-dateutil",
        "louie"
    ],
    scripts=["bin/pysmhs"]
    )



import logging

from twisted.internet import serialport, reactor
from twisted.internet import defer, task
from twisted.internet.protocol import ClientFactory

from pymodbus.factory import ClientDecoder
from pymodbus.client.async import ModbusClientProtocol
from pymodbus.transaction import ModbusAsciiFramer as ModbusFramer

from serial import PARITY_NONE, PARITY_EVEN, PARITY_ODD
from serial import STOPBITS_ONE, STOPBITS_TWO
from serial import FIVEBITS, SIXBITS, SEVENBITS, EIGHTBITS

from abstracthandler import AbstractHandler

logger = logging.getLogger('plchandler')

pymodbus_logger = logging.getLogger('pymodbus')
pymodbus_logger.setLevel(logging.ERROR)

class SMHSProtocol(ModbusClientProtocol):

    def __init__(self, framer, pol_list, reader, writepool):
        ''' Initializes our custom protocol

        :param framer: The decoder to use to process messages
        :param endpoint: The endpoint to send results to
        '''
        ModbusClientProtocol.__init__(self, framer)
        self.pol_list = pol_list
        self.reader = reader
        self.writepool = writepool
        self.loop = task.LoopingCall(self.fetch_holding_registers)

    def connectionMade(self):
        logger.debug("Connected to ModBus")
        super(SMHSProtocol, self).connectionMade()
        self.loop.start(0)

    # def connectionLost(self, reason):
        # self.loop.stop()
        # super(SMHSProtocol, self).connectionLost(reason)
        # logger.debug('lost')

    # @defer.inlineCallbacks
    # def read_reg(self):
        # #Write counter threshold
        # reg = (4598, 1)
        # logger.debug('start read')
        # result = yield self.read_holding_registers(*reg)
        # logger.debug('readed {}'.format(result))
        # #Write polling tag
        # # yield self.write_coil(2057, 0xFF00)
        # while (1):
            # try:
                # yield self.fetch_holding_registers()
                # logger.debug('readed'.format(result))
            # except Exception as e:
                # logger.exception('')

    @defer.inlineCallbacks
    def fetch_holding_registers(self):
        # logger.debug('read registers from {}'.format(self.pol_list))
        try:
            address_map = self.pol_list.get("inputc", None)
            if address_map:
                for register in address_map:
                    response = yield self.read_holding_registers(*register)
                    val = {}
                    for i in range(0, register[1]):
                        val[register[0] + i] = response.getRegister(i)
                    self.reader(val, "inputc")

            address_map = self.pol_list.get("output", None)
            if address_map:
                for register in address_map:
                    response = yield self.read_coils(*register)
                    val = {}
                    for i in range(0, register[1]):
                        val[register[0] + i] = response.getBit(i)
                    self.reader(val, "output")
        except Exception:
            logger.exception("can't fetch registers")

    # def write_tags(self, response):
        # d = None
        # if len(self.writepool):
            # logger.debug("writepool len = %d" % len(self.writepool))
            # for x in self.writepool.keys():
                # val = int(self.writepool.pop(x))
                # logger.debug(
                    # "writting tag %s to %d" % (x, val))
                # if val:
                    # val = 0xFF00
                # else:
                    # val = 0x0000
                # if not d:
                    # d = self.write_tag(response, x, val)
                # else:
                    # d.addCallback(self.write_tag, addr=x, value=val)
        # if not d:
            # self.write_polling_tag(response)
        # else:
            # d.addCallback(self.write_polling_tag)

    @defer.inlineCallbacks
    def write_tag(self, addr, value):
        logger.debug('write tag {} with value {} to plc'.format(addr, value))
        if value:
            val = 0xFF00
        else:
            val = 0x0000
        try:
            result = yield self.write_coil(addr, val)
            if not result.value:
                raise ValueError("result not true")
        except Exception:
            logger.exception(
                "while write to coil {} value {}".format(addr, val))
        defer.returnValue(result)



class SMHSFactory(ClientFactory):

    protocol = SMHSProtocol

    def __init__(self, framer, pol_list, reader, writepool):
        self.framer = framer
        self.pol_list = pol_list
        self.reader = reader
        self.writepool = writepool
        self.proto = None

    def buildProtocol(self, _):
        proto = self.protocol(
            self.framer,
            self.pol_list, self.reader, self.writepool)
        self.proto = proto
        proto.factory = self
        return proto


class SerialModbusClient(serialport.SerialPort):

    def __init__(self, factory, *args, **kwargs):
        protocol = factory.buildProtocol(None)
        self.decoder = ClientDecoder()
        serialport.SerialPort.__init__(self, protocol, *args, **kwargs)


class plchandler(AbstractHandler):

    def __init__(self, parent=None, params={}):
        AbstractHandler.__init__(self, parent, params)
        logger.info("Init async_plchandler")
        serverconfig = params["server"]
        self.serial_port = params["port"]
        self.pollint = serverconfig["pollingTimeout"]
        self.packetSize = int(serverconfig["packetSize"])
        self.tagslist = {}
        self.writepool = {}
        self._inputctags = {}
        self._inputtag_threshold = int(serverconfig["counter_threshold"])
        #fill tagslist with tags from all types
        for tagtype in self.config:
            self.tagslist.update(self.config[tagtype])
        #fill address list
        self.full_address_list = {}
        # logger.debug(self.tagslist)
        for x in self.tagslist:
            if "address" in self.tagslist[x]:
                address = self.tagslist[x]["address"]
                self.full_address_list[int(address)] = x
        # for tag in self.tagslist.keys():
            # self._tags[tag] = 0
        # logger.debug(self._tags)
        logger.debug("Full address list - %s" % self.full_address_list)

    def settag(self, name, value):
        logger.debug("set tag %s to %s" % (name, value))
        # self.writepool[int(self.tagslist[name]["address"])] = value
        address = int(self.tagslist[name]['address'])
        self.factory.proto.write_tag(address, value)

    def _generate_address_map(self, addressList):
        '''
        generate addressMap based on the addressList
        addressMap is dictionary with key = startaddress to read
        and value = number of bits need to read
        '''
        keylist = addressList.keys()
        maxAddress = int(max(keylist))
        minAddress = int(min(keylist))
        s = maxAddress - minAddress + 1
        c, d = divmod(s, self.packetSize)
        l = maxAddress - d + 1
        addressMap = []
        for x in range(0, c):
            curAddress = minAddress + self.packetSize * x
            addressMap.append((curAddress, self.packetSize,))
        if (d > 0):
            addressMap.append((l, d,))
        return tuple(addressMap)

    def reader(self, register, t):
        for addr in register:
            if addr in self.full_address_list:
                tagname = self.full_address_list[addr]
                tagstate = int(register[addr])
                if t == "inputc":
                    self.__addinputctag(tagname, tagstate)
                else:
                    self.__addtag(tagname, tagstate)

    def __addtag(self, tag, value):
        '''
        check if this tag in polling cache
        and if not add it
        and if tag change it value since last pooling
        add it to events list
        '''
        if tag in self._tags:
            super(plchandler, self).settag(tag, value)
        else:
            logger.debug('add tag {} with value {}'.format(tag, value))
            self._tags[tag] = value

    def __addinputctag(self, tag, value):
        '''
        add event for input registers
        count diff from last value and send value
        this tag would produce as normal input tag
        '''
        if tag in self._inputctags:
            lastval = self._inputctags[tag]
            if lastval != value:
                if value > lastval:
                    for x in range(lastval + 1, value + 1):
                        self.sendevents(tag, x & 1)
                else:
                    dif = self._inputtag_threshold - lastval + value
                    for x in range(lastval + 1, lastval + dif + 1):
                        self.sendevents(tag, x & 1)

        self._inputctags[tag] = value
        self._tags[tag] = value & 1

    def start(self):
        AbstractHandler.start(self)
        framer = ModbusFramer(ClientDecoder())
        pol_list = {}
        for t in self.config.keys():
            if t in ["output", "input", "inputc"]:
                address_list = {}
                for x in self.config[t]:
                    address = self.tagslist[x]["address"]
                    address_list[address] = x
                pol_list[t] = self._generate_address_map(address_list)
        logger.debug(pol_list)
        factory = SMHSFactory(
            framer, pol_list, self.reader, self.writepool)
        self.factory = factory
        SerialModbusClient(
            factory, self.serial_port['name'],
            reactor, baudrate=9600,
            parity=PARITY_EVEN, bytesize=SEVENBITS,
            stopbits=STOPBITS_TWO, timeout=0)

    def stop(self):
        AbstractHandler.stop(self)



import unittest
from pysmhs import datehandler


class TestDatehandler(unittest.TestCase):

    def setUp(self):
        configfile = "test/dateconfig.txt"
        self.d = datehandler.datehandler(params={'configfile': configfile})
        self.d.start()

    def testDate(self):
        print("")
        print(self.d.tags)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
'''
Web server handler
'''
from abstracthandler import AbstractHandler
from twisted.web import server, resource
from twisted.internet import reactor
from twisted.web.resource import Resource
from twisted.web.static import File
import cgi
from jinja2 import Environment, PackageLoader
from collections import OrderedDict
from uuid import uuid4

import logging
from pkg_resources import resource_filename

logger = logging.getLogger('webhandler')


class webhandler(AbstractHandler):

    '''Web server handler'''

    port = None

    def __init__(self, parent=None, params={}):
        self.cachemax = 255
        self.eventcache = OrderedDict()
        self.params = params
        AbstractHandler.__init__(self, parent, params)
        logger.info("Init web handler")
        #  resource = File(params["wwwPath"])
        root = File(resource_filename('pysmhs', 'www'))
        #  root.putChild("www", resource)
        root.putChild("get", smhs_web(parent))
        #root.putChild("mon", monitor(self.eventcache))
        self.site = server.Site(root)

    def loadtags(self):
        pass

    def process(self, signal, events):
        for event in events:
            if len(self.eventcache) == self.cachemax:
                self.eventcache.popitem(last=False)
            self.addevent(event)

    def addevent(self, event):
        token = uuid4().bytes.encode("base64")
        self.eventcache[token] = event

    def start(self):
        AbstractHandler.start(self)
        self.port = reactor.listenTCP(int(self.params["port"]), self.site)

    def stop(self):
        AbstractHandler.stop(self)
        if self.port:
            self.port.stopListening()


class smhs_web(resource.Resource):
    isLeaf = True
    action_get_json = "getJson"
    actionStopServer = "stopServer"
    action_list_tags = "listTags"
    action_set_tag = "setTag"

    def __init__(self, parent):
        env = Environment(loader=PackageLoader('pysmhs', 'www/templates'))
        self.listtags_template = env.get_template('listtags_template.html')
        self.parent = parent
        resource.Resource.__init__(self)

    def render_GET(self, request):
        if ("action" in request.args):
            if (request.args["action"][0] == self.action_get_json):
                html = "{ \"tags\":{"
                coils = self.parent.tags
                for x in coils:
                    html += "\"" + x + "\":\"" + str(coils[x]) + "\","
                html += "} }"
                return html
            elif (request.args["action"][0] == self.action_list_tags):
                tags = self.parent.tags
                od = {}
                last_handler = ""
                for tag in sorted(tags):
                    current_handler = tag.split('_')[0]
                    if current_handler != last_handler:
                        last_handler = current_handler
                    tag_name = tag.split('_')[1]
                    od.setdefault(last_handler, {})[tag_name] = str(tags[tag])
                return str(self.listtags_template.render(title=u'Tag list',
                                                         description='here',
                                                         tags=od))
            elif (request.args["action"][0] == self.action_set_tag):
                l = request.args
                del l['action']
                html = ''
                for tag in l:
                    self.parent.settag(tag, int(l[tag][0]))
                    html += "setting %s to %s" % (tag, l[tag][0])
                return html
            else:
                if (request.args["action"][0] == self.actionStopServer):
                    self.parent.stop()
                    return "Close"
        return "unknown url"

    def render_POST(self, request):
        for x in request.args:
            if (cgi.escape(request.args[x][0]) == "1"):
                self.parent.settag(x, 1)
            else:
                self.parent.settag(x, 0)

class monitor(resource.Resource):
    isLeaf = True

    def __init__(self, eventcache):
        env = Environment(loader=PackageLoader('www', 'templates'))
        self.eventcache = eventcache
        self.monitor_template = env.get_template('monitor_template.html')
        resource.Resource.__init__(self)

    def render_GET(self, request):
        return str(self.monitor_template.render(
            title=u'Monitor', description='here', events=self.eventcache))
